%%
% Authors:
%   Brian Hackett  <bhackett@stanford.edu>
%
% Copyright (c) 2006,
%   The Board of Trustees of The Leland Stanford Junior University
% All rights reserved.
%
% Redistribution and use in source and binary forms, with or without
% modification, are permitted provided that the following conditions are met:
% 
% 1. Redistributions of source code must retain the above copyright notice,
% this list of conditions and the following disclaimer.
% 
% 2. Redistributions in binary form must reproduce the above copyright notice,
% this list of conditions and the following disclaimer in the documentation
% and/or other materials provided with the distribution.
% 
% 3. The names of the contributors may not be used to endorse or promote
% products derived from this software without specific prior written
% permission.
% 
% THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS"
% AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
% IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE
% ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT OWNER OR CONTRIBUTORS BE
% LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR
% CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF
% SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS
% INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN
% CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE)
% ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE
% POSSIBILITY OF SUCH DAMAGE.
%%

% per-global phase of aliasing.clp. scan global static initializers for new
% global and type invariants, and fill in sum_glob->spoints according to any
% composite type invariants discovered during per-function analysis

import "../memory/traces.clp".
import "aliasinginitui.clp".

analyze session_name("cil_glob").
analyze analysis_name("aliasinit").

predicate omit_comp_spoints(in C:string,in SR:t_trace,in TR:t_trace).
predicate drop_spoints(in S:t_trace,in T:t_trace).

?drop_spoints(S,T), trace_sub(S,STR,RS), trace_type(s_func,STR,y_comp{C}),
    sum_comp(C)->spoly(_,RS,_), +drop_spoints(S,T).

% RULES

predicate init_spoints(S:t_trace,T:t_trace,E:c_exp).

% analogous to memory.clp version, no per-pp sensitivity though
predicate trace_offset(in RT:t_trace,in OFF:c_offset,T:t_trace).
cil_off_none(OFF), +trace_offset(RT,OFF,RT).
cil_off_field(OFF,FX,NOFF), cil_field_name(FX,F,_,_,_),
    ?trace_offset(RT,OFF,_),
    trace_type(s_func,RT,y_comp{C}), trace_simplify(s_func,fld{RT,F,C},FT),
    trace_offset(FT,NOFF,T), +trace_offset(RT,OFF,T).
cil_off_index(OFF,_,NOFF), ?trace_offset(RT,OFF,_),
    trace_offset(RT,NOFF,T), +trace_offset(RT,OFF,T).

predicate init_eval(in E:c_exp,T:t_trace).
cil_exp_addr(E,LV), cil_lval_var(LV,X,OFF), cil_var_name(X,DG,_),
    trace_offset(root{glob{DG}},OFF,T), +init_eval(E,T).
cil_exp_cast(E,CE,_), init_eval(CE,T), +init_eval(E,T).

% points-to relationships from static initializers
predicate static_initializer(t_trace,c_init).
static_initializer(T,INIT), cil_init_single(INIT,E), init_eval(E,DT),
    ~drop_spoints(T,DT), +init_spoints(T,DT,E).
static_initializer(T,INIT), cil_init_cmpnd_field(INIT,FLD,SINIT),
    cil_field_name(FLD,F,_,_,_), cil_init_cmpnd_type(INIT,TYP),
    cil_type_comp(TYP,C), +static_initializer(fld{T,F,C},SINIT).
static_initializer(T,INIT), cil_init_cmpnd_index(INIT,_,SINIT),
    +static_initializer(T,SINIT).

cil_glob(G), +cil_init(G)->merge_preds(),
    cil_var_init(_,INIT), +static_initializer(root{glob{G}},INIT).

% now add the invariant, using the composite type if available

predicate use_comp_spoints(S:t_trace,T:t_trace).

init_spoints(fld{FS,F,C},T,E), ~omit_comp_spoints(C,fld{empty,F,C},T),
    +add_spoints_display(td_comp_points{C,FS},fld{FS,F,C},T,E),
    +use_comp_spoints(fld{FS,F,C},T).

cil_glob(G), init_spoints(S,T,E), ~use_comp_spoints(S,T),
    +add_spoints_display(td_glob_points{G},S,T,E).

%+omit_comp_spoints(_,_,_).
