%%
% Authors:
%   Brian Hackett  <bhackett@stanford.edu>
%
% Copyright (c) 2006,
%   The Board of Trustees of The Leland Stanford Junior University
% All rights reserved.
%
% Redistribution and use in source and binary forms, with or without
% modification, are permitted provided that the following conditions are met:
% 
% 1. Redistributions of source code must retain the above copyright notice,
% this list of conditions and the following disclaimer.
% 
% 2. Redistributions in binary form must reproduce the above copyright notice,
% this list of conditions and the following disclaimer in the documentation
% and/or other materials provided with the distribution.
% 
% 3. The names of the contributors may not be used to endorse or promote
% products derived from this software without specific prior written
% permission.
% 
% THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS"
% AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
% IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE
% ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT OWNER OR CONTRIBUTORS BE
% LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR
% CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF
% SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS
% INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN
% CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE)
% ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE
% POSSIBILITY OF SUCH DAMAGE.
%%

% generate DOT graphs for the generated CFG on each function
% importing this file will add, for each function foo, a file foo_cfg.dot
% which can be viewed using dotty etc.

import "cilcfg.clp".

using dotty.

% add to label points in the graph as FILE:LINE rather than PP
predicate dot_point_locations().

% use NAME instead of foo_cfg for the dotty file
predicate dot_cfg_name(NAME:string).

% dotty graph

% get the graph ID to use for the current function
predicate cfgid(dotgraph).
dot_cfg_name(NAME), dotty_graph(NAME,true,GID), +cfgid(GID).
~dot_cfg_name(_), cil_curfn(FN), str_cat(FN,"_cfg",FNG),
    dotty_graph(FNG,true,GID), +cfgid(GID).

% color all the graph nodes using global attributes
cfgid(GID), +dotty_attr(GID,"node","color","green"),
            +dotty_attr(GID,"node","style","filled").

% add labels for special CFG nodes
type label_info ::=
    lab_edge{P,N}
  | lab_node{P,N}.
cfgid(GID), cfgn_final(P,N), tostring(N,NS),
    +dotty_edge(GID,lab_edge{P,N},lab_node{P,N},P),
    +dotty_attr(GID,lab_node{P,N},"shape","box"),
    +dotty_attr(GID,lab_node{P,N},"color","cyan"),
    +dotty_attr(GID,lab_node{P,N},"label",NS).

% add edges and label attributes for CFG edges
type edge_info ::= dot_edge{P0,P1}.
cfgid(GID), cfge_final(P0,P1,_), 
    +dotty_edge(GID,dot_edge{P0,P1},P0,P1),
    \/cfge_final(P0,P1,E):list_all(E,Es), tostring(Es, Label),
    +dotty_attr(GID,dot_edge{P0,P1},"label",Label).

dot_point_locations(), cfgid(GID), cfge_final(P0,P1,_),
    point_location(P0,FILE0,LINE0), point_location(P1,FILE1,LINE1),
    location_string(FILE0,LINE0,S0), +dotty_attr(GID,P0,"label",S0),
    location_string(FILE1,LINE1,S1), +dotty_attr(GID,P1,"label",S1).
