%%
% Authors:
%   Brian Hackett  <bhackett@stanford.edu>
%
% Copyright (c) 2006,
%   The Board of Trustees of The Leland Stanford Junior University
% All rights reserved.
%
% Redistribution and use in source and binary forms, with or without
% modification, are permitted provided that the following conditions are met:
% 
% 1. Redistributions of source code must retain the above copyright notice,
% this list of conditions and the following disclaimer.
% 
% 2. Redistributions in binary form must reproduce the above copyright notice,
% this list of conditions and the following disclaimer in the documentation
% and/or other materials provided with the distribution.
% 
% 3. The names of the contributors may not be used to endorse or promote
% products derived from this software without specific prior written
% permission.
% 
% THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS"
% AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
% IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE
% ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT OWNER OR CONTRIBUTORS BE
% LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR
% CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF
% SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS
% INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN
% CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE)
% ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE
% POSSIBILITY OF SUCH DAMAGE.
%%

% Dumb as dirt modelling of integer operations

% Operations defined here and in other int modelling programs are:
% exp_const
% exp_unop and exp_binop where the op is not logic_op or pointer_op
% exp_cast with int result
% boolean evaluation of integer expressions

import "traces.clp".
import "../base/cilcfg.clp".

% ANALYSIS REPRESENTATIONS

import "memory.clp".

% int representation, result of evaluating some exp E at P
type t_nrep ::= unint{P:pp,E:c_exp}.

% guards are either uninterpreted ints or are unconstrained bits from upstream
% analyses
type g_bit ::=
    ar_int{t_nrep}
  | ar_path{pp,pp}
  | ar_entry{pp,t_trace,t_trace}
  | ar_exit{sum,pp,t_trace,t_trace}.

% RULES

% implement satisfiability predicates for upstream analyses
% note that asn_value will always fail if this file is used
type g_guard_asn = bval_asn[g_bit].
?guard_sat(G), #sat(G), +guard_sat(G).
?guard_sat_asn(G,ASN), #satasn(G,ASN), +guard_sat_asn(G,ASN).
?asn_guard(ASN,G), #asng(ASN,G), +asn_guard(ASN,G).

% implement unconstrained bits for upstream analyses
?path_bit(P0,P1,_), #id_g(ar_path{P0,P1},G), +path_bit(P0,P1,G).
?entry_bit(P,S,T,_), #id_g(ar_entry{P,S,T},G), +entry_bit(P,S,T,G).
?exit_bit(SUM,P,S,T,_), #id_g(ar_exit{SUM,P,S,T},G), +exit_bit(SUM,P,S,T,G).

% just evaluate int and float expressions to an unint. lval expressions
% are handled by memory.clp, casts handled below
exp_tint(E,_), ~cil_exp_cast(E,_,_), ~cil_exp_lval(E,_),
    ?eval(P,E,_,_), ~omit_eval(P,E),
    #bool_g(true,G), +eval(P,E,nrep{unint{P,E}},G).
exp_tflt(E,_), ~cil_exp_lval(E,_), ?eval(P,E,_,_), ~omit_eval(P,E),
    #bool_g(true,G), +eval(P,E,nrep{unint{P,E}},G).

% handle scalar to pointer casts and * to scalar casts
cil_exp_cast(E,CE,_), exp_tptr(E), ~exp_tptr(CE), ?eval(P,E,_,_), 
    ~omit_eval(P,E), eval(P,CE,V,G), +eval(P,E,V,G).
cil_exp_cast(E,CE,_), ~exp_tptr(E), ?eval(P,E,_,_),
    ~omit_eval(P,E), eval(P,CE,V,G), +eval(P,E,V,G).

% get an unconstrained value for &((type*)0->field)
cil_exp_x_intval(MCE,0), cil_exp_cast(ME,MCE,_), cil_lval_mem(LV,ME,_),
    cil_exp_addr(AE,LV), cil_exp_cast(E,AE,_),
    ?eval(P,E,_,_), ~omit_eval(P,E),
    #bool_g(true,G), +eval(P,E,nrep{unint{P,E}},G).

% convert int expressions to guards by getting a fresh boolean variable
exp_tint(E,_), ~exp_boolean(E), ?beval(P,E,_,_), ~omit_beval(P,E),
    #bool_g(true,G), #id_g(ar_int{unint{P,E}},BG), +beval(P,E,BG,G).
compare_op(OP), cil_exp_binop(E,OP,_,_,_), ?beval(P,E,_,_), ~omit_beval(P,E),
    #bool_g(true,G), #id_g(ar_int{unint{P,E}},BG), +beval(P,E,BG,G).
