%%
% Authors:
%   Brian Hackett  <bhackett@stanford.edu>
%
% Copyright (c) 2006,
%   The Board of Trustees of The Leland Stanford Junior University
% All rights reserved.
%
% Redistribution and use in source and binary forms, with or without
% modification, are permitted provided that the following conditions are met:
% 
% 1. Redistributions of source code must retain the above copyright notice,
% this list of conditions and the following disclaimer.
% 
% 2. Redistributions in binary form must reproduce the above copyright notice,
% this list of conditions and the following disclaimer in the documentation
% and/or other materials provided with the distribution.
% 
% 3. The names of the contributors may not be used to endorse or promote
% products derived from this software without specific prior written
% permission.
% 
% THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS"
% AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
% IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE
% ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT OWNER OR CONTRIBUTORS BE
% LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR
% CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF
% SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS
% INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN
% CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE)
% ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE
% POSSIBILITY OF SUCH DAMAGE.
%%

% baked-in code-base specific information to use during memory analysis.
% imported directly by memory.clp

% linux

% special case 'reverse field' pointer arithmetic from container_of:
% (ptr - (int)&((type*)0)->field)
cil_exp_binop(E,b_minuspi,LE,RE,_),
    cil_exp_cast(LE,LCE,_),
    cil_exp_cast(RE,RCE,_), cil_exp_addr(RCE,LV), cil_lval_mem(LV,ME,OFF),
    cil_exp_cast(ME,CME,_), cil_exp_x_intval(CME,0),
    +omit_eval(_,E), ?eval(P,E,_,_), eval(P,LCE,trace{T},G),
    trace_rev_offset(T,OFF,FT), +eval(P,E,trace{FT},G).

% get names for untyped hidden fields $comp__N
predicate str_hidden_field(in COMP:string,in N:int,FLD:string).
?str_hidden_field(COMP,N,_), tostring(N,NS),
    str_cat("$",COMP,S0), str_cat(S0,"$",S1), str_cat(S1,NS,FLD),
    +str_hidden_field(COMP,N,FLD).

% special case 'hidden field' pointer arithmetic: adding a fixed offset N
% to a structure pointer where N is larger than the size of the structure
% yields a hidden field for the comp type and offset
cil_exp_binop(E,b_pluspi,LE,RE,_),
    cil_exp_cast(LE,LCE,NTYP), exp_type(LCE,OTYP),
    cil_type_ptr(NTYP,DNTYP), cil_type_ptr(OTYP,DOTYP), cil_type_comp(DOTYP,C),
    cil_type_x_bytes(DOTYP,WIDTH), cil_type_x_bytes(DNTYP,1),
    cil_exp_x_intval(RE,N), int_ge(N,WIDTH), str_hidden_field(C,N,F),
    +omit_eval(_,E), ?eval(P,E,_,_),
    eval(P,LE,trace{T},G), +eval(P,E,trace{fld{T,F,C}},G).
cil_exp_binop(E,b_pluspi,LE,RE,_), cil_exp_cast(_,E,_),
    exp_type(LE,TYP), cil_type_ptr(TYP,DTYP), cil_type_comp(DTYP,C),
    cil_type_x_bytes(DTYP,N), cil_exp_x_intval(RE,1), str_hidden_field(C,N,F),
    +omit_eval(_,E), ?eval(P,E,_,_),
    eval(P,LE,trace{T},G), +eval(P,E,trace{fld{T,F,C}},G).

% the alloc_percpu and per_cpu_ptr macros in linux work by using the bitwise
% negation of pointers and dereferencing those. incredibly foul, and screws
% up the memory analysis if not special-cased
cil_exp_unop(CE,u_bnot,_,_), cil_exp_cast(E,CE,TYP), cil_type_ptr(TYP,_),
    tostring(E,ES), +temp_type(ES,y_ptr{y_void}),
    +omit_eval(_,E), ?eval(P,E,_,_), #bool_g(true,G),
    +eval(P,E,trace{root{temp{ES,"per_cpu"}}},G).

% understand __builtin_expect, which just returns its first argument
dircall(I,"__builtin_expect"), icall(P0,P1,I),
    cil_instr_call_arg(I,0,E), callret(I,S),
    #bool_g(false,FG), +inst_transfer_necessary(s_call{I},P0,root{S},FG),
    ?vmerge(P1,root{S},_,_), eval(P0,E,T,G), +vmerge(P1,root{S},T,G).
