/*
 * Authors:
 *   Peter Hawkins  <hawkinsp@cs.stanford.edu>
 *
 * Copyright (c) 2005-2006,
 *   The Board of Trustees of The Leland Stanford Junior University
 * All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions are met:
 * 
 * 1. Redistributions of source code must retain the above copyright notice,
 * this list of conditions and the following disclaimer.
 * 
 * 2. Redistributions in binary form must reproduce the above copyright notice,
 * this list of conditions and the following disclaimer in the documentation
 * and/or other materials provided with the distribution.
 * 
 * 3. The names of the contributors may not be used to endorse or promote
 * products derived from this software without specific prior written
 * permission.
 * 
 * THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS"
 * AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
 * IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE
 * ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT OWNER OR CONTRIBUTORS BE
 * LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR
 * CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF
 * SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS
 * INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN
 * CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE)
 * ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE
 * POSSIBILITY OF SUCH DAMAGE.
 */

/* mlstp.cc
 * ML bindings for the STP theorem prover.
 */

#include <cstdio>
#include <stp/c_interface.h>

using namespace std;
extern "C" {
#include <caml/mlvalues.h>
#include <caml/alloc.h>
#include <caml/memory.h>
#include <caml/signals.h>
}

static VC vc;

/* Hack: we use the following dummy class to run our various initialization
 * processes by creating a single static instance of the class and running
 * our initializers from the constructor */
static class CVCInit {
public:
    CVCInit() { vc = vc_createValidityChecker(); }
} VCInit;

static void expr_finalize(value v)
{
    // XXX: Implement me
}

#define Expr_val(v) (*((Expr *)Data_custom_val(v)))

inline value box_stpexpr(Expr e) {
    value v = caml_alloc_final(2, expr_finalize, 1, 100);
    Expr_val(v) = e;
    return v;
}

extern "C" {

value mlstp_new_bvar(value unit)
{
    char namebuffer[100];
    static int num_bvars = 0;

    CAMLparam1(unit);
    CAMLlocal1(v);
    snprintf(namebuffer, sizeof(namebuffer), "b%d\n", num_bvars++);
    v = box_stpexpr(vc_varExpr(vc, namebuffer, vc_boolType(vc)));
    CAMLreturn(v);
}

value mlstp_true(value unit)
{
    CAMLparam1(unit);
    CAMLlocal1(v);
    v = box_stpexpr(vc_trueExpr(vc));
    CAMLreturn(v);
}

value mlstp_false(value unit)
{
    CAMLparam1(unit);
    CAMLlocal1(v);
    v = box_stpexpr(vc_falseExpr(vc));
    CAMLreturn(v);
}


#define STP_UNOP(MLFN, STPFN) \
value MLFN(value w)                                 \
{                                                   \
    CAMLparam1(w);                               \
    CAMLlocal1(v);                                  \
    v = box_stpexpr(STPFN(vc, Expr_val(w)));        \
    CAMLreturn(v);                                  \
}

#define STP_BINOP(MLFN, STPFN) \
value MLFN(value a, value b)                                \
{                                                           \
    CAMLparam2(a,b);                                        \
    CAMLlocal1(v);                                          \
    v = box_stpexpr(STPFN(vc, Expr_val(a), Expr_val(b)));   \
    CAMLreturn(v);                                          \
}

STP_UNOP(mlstp_lnot, vc_notExpr);

STP_BINOP(mlstp_land, vc_andExpr);
STP_BINOP(mlstp_lor,  vc_orExpr);

value mlstp_vc_push(value unit)
{
    CAMLparam1(unit);
    vc_push(vc);
    CAMLreturn(Val_unit);
}

value mlstp_vc_pop(value unit)
{
    CAMLparam1(unit);
    vc_pop(vc);
    CAMLreturn(Val_unit);
}

value mlstp_vc_assert(value vb)
{
    CAMLparam1(vb);
    vc_assertFormula(vc, Expr_val(vb));
    CAMLreturn(Val_unit);
}

value mlstp_vc_query(value qe)
{
    int ret;
    CAMLparam1(qe);
    ret = vc_query(vc, Expr_val(qe));
    CAMLreturn((ret ? Val_true : Val_false));
}

value mlstp_get_bval(value qe)
{
    int ret;
    CAMLparam1(qe);
    //ret = vc_query(vc, Expr_val(qe));
    CAMLreturn(Val_int(0));
}

} /* extern "C" */
