(*
 * Authors:
 *   Peter Hawkins  <hawkinsp@cs.stanford.edu>
 *
 * Copyright (c) 2005-2006,
 *   The Board of Trustees of The Leland Stanford Junior University
 * All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions are met:
 * 
 * 1. Redistributions of source code must retain the above copyright notice,
 * this list of conditions and the following disclaimer.
 * 
 * 2. Redistributions in binary form must reproduce the above copyright notice,
 * this list of conditions and the following disclaimer in the documentation
 * and/or other materials provided with the distribution.
 * 
 * 3. The names of the contributors may not be used to endorse or promote
 * products derived from this software without specific prior written
 * permission.
 * 
 * THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS"
 * AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
 * IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE
 * ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT OWNER OR CONTRIBUTORS BE
 * LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR
 * CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF
 * SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS
 * INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN
 * CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE)
 * ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE
 * POSSIBILITY OF SUCH DAMAGE.
 *)

(* stp.ml
 * OCaml interface to Vijay Ganesh's SAT-based Theorem Prover (STP) *)

(* The type of STP types *)
type typ

(* The type of STP boolean expressions *)
type bexpr

(* The type of STP expressions *)
(*type expr *)

(* Push a checkpoint onto the STP context stack *)
external vc_push : unit -> unit = "mlstp_vc_push"
external vc_pop  : unit -> unit = "mlstp_vc_pop"

(* Assert a formula in the current solver context *)
external vc_assert : bexpr -> unit = "mlstp_vc_assert"

(* Test an expression for validity *)
external vc_query : bexpr -> bool = "mlstp_vc_query"

(* Three-valued logic value *)
type tvlv =
    False
  | True
  | Unknown 

external vc_get_bval : bexpr -> tvlv = "mlstp_get_bval" 


(* Boolean expressions *)
external b_newvar : unit -> bexpr = "mlstp_new_bvar"

external get_b_true  : unit -> bexpr = "mlstp_true"
external get_b_false : unit -> bexpr = "mlstp_false"
let b_true = get_b_true ()
let b_false = get_b_false ()

external b_and   : bexpr -> bexpr -> bexpr = "mlstp_land"
external b_or    : bexpr -> bexpr -> bexpr = "mlstp_lor"
external b_not   : bexpr -> bexpr = "mlstp_lnot"


