(*
 * Authors:
 *   Brian Hackett  <bhackett@stanford.edu>
 *
 * Copyright (c) 2005-2006,
 *   The Board of Trustees of The Leland Stanford Junior University
 * All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions are met:
 * 
 * 1. Redistributions of source code must retain the above copyright notice,
 * this list of conditions and the following disclaimer.
 * 
 * 2. Redistributions in binary form must reproduce the above copyright notice,
 * this list of conditions and the following disclaimer in the documentation
 * and/or other materials provided with the distribution.
 * 
 * 3. The names of the contributors may not be used to endorse or promote
 * products derived from this software without specific prior written
 * permission.
 * 
 * THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS"
 * AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
 * IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE
 * ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT OWNER OR CONTRIBUTORS BE
 * LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR
 * CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF
 * SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS
 * INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN
 * CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE)
 * ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE
 * POSSIBILITY OF SUCH DAMAGE.
 *)

(* heap array based priority queue implementation *)

exception Empty

module V = Vector
type 'a t = ('a -> 'a -> int) * 'a V.t

let higher fn heap  pos0 pos1 = fn (V.get heap pos0) (V.get heap pos1) < 0
let swap heap pos0 pos1 =
  let tmp = V.get heap pos0 in
    V.set heap pos0 (V.get heap pos1);
    V.set heap pos1 tmp

let create fn = (fn, V.create 100)

let insert (fn,heap) a =
  (* add at end of vector and percolate up, O(log n) *)
  let rec percolate pos =
    let parpos = (pos-1)/2 in
      if pos > 0 && higher fn heap pos parpos then
        (swap heap pos parpos; percolate parpos) in
  percolate (V.append heap a)

let extract (fn,heap) =
  (* swap with end of vector and percolate down, O(log n) *)
  let size = V.size heap - 1 in
  let rec percolate pos =
    let lpos = pos*2+1 in
    let rpos = lpos+1 in
    if rpos < size && higher fn heap rpos lpos then
      (if higher fn heap rpos pos then (swap heap rpos pos; percolate rpos))
    else if lpos < size then
      (if higher fn heap lpos pos then (swap heap lpos pos; percolate lpos))
    else () in
  if size = -1 then raise Empty
  else let a = V.get heap 0 in
    (* verify we got the minimal element
    for i = 1 to (V.size heap - 1) do
      assert (not (higher fn heap i 0));
    done;
    *)
    swap heap 0 size;
    V.shrink heap size;
    percolate 0; a

(*
type 'a t = ('a -> 'a -> int) * 'a list ref

let create fn = (fn, ref [])

let insert (fn,list) a =
  list := List.merge fn [a] !list

let extract (fn,list) =
  try let a = List.hd !list in list := List.tl !list; a
  with _ -> raise Empty
*)
